#!/usr/bin/env python3
"""
Advanced Network Monitor
Parameters:
    - IP: Interface IP to monitor
Additional parameters in comments:
    - interval: Monitoring interval in seconds
    - threshold: Alert threshold in percent
"""
import psutil
import time
import argparse
from datetime import datetime
import json
import socket
import threading
import queue
import logging
from pathlib import Path

class NetworkMonitor:
    def __init__(self, interface_ip, interval=1, threshold=80):
        self.interface_ip = interface_ip
        self.interval = interval
        self.threshold = threshold
        self.interface_name = self._get_interface_name()
        self.stop_event = threading.Event()
        self.alert_queue = queue.Queue()
        self.metrics = {}
        self.setup_logging()
        
    def setup_logging(self):
        """Set up logging configuration"""
        log_dir = Path("network_logs")
        log_dir.mkdir(exist_ok=True)
        
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s',
            handlers=[
                logging.FileHandler(log_dir / "network_monitor.log"),
                logging.StreamHandler()
            ]
        )
        
    def _get_interface_name(self):
        """Get interface name from IP address"""
        for iface, addrs in psutil.net_if_addrs().items():
            for addr in addrs:
                if addr.address == self.interface_ip:
                    return iface
        raise ValueError(f"No interface found with IP {self.interface_ip}")
        
    def collect_interface_metrics(self):
        """Collect current interface metrics"""
        try:
            io_counters = psutil.net_io_counters(pernic=True)[self.interface_name]
            stats = psutil.net_if_stats()[self.interface_name]
            
            return {
                'bytes_sent': io_counters.bytes_sent,
                'bytes_recv': io_counters.bytes_recv,
                'packets_sent': io_counters.packets_sent,
                'packets_recv': io_counters.packets_recv,
                'errin': io_counters.errin,
                'errout': io_counters.errout,
                'dropin': io_counters.dropin,
                'dropout': io_counters.dropout,
                'speed': stats.speed,
                'mtu': stats.mtu,
                'is_up': stats.isup
            }
        except Exception as e:
            logging.error(f"Error collecting metrics: {str(e)}")
            return None
            
    def calculate_rates(self, old_metrics, new_metrics, interval):
        """Calculate rate metrics"""
        if not old_metrics or not new_metrics:
            return None
            
        return {
            'bytes_sent_per_sec': (new_metrics['bytes_sent'] - old_metrics['bytes_sent']) / interval,
            'bytes_recv_per_sec': (new_metrics['bytes_recv'] - old_metrics['bytes_recv']) / interval,
            'packets_sent_per_sec': (new_metrics['packets_sent'] - old_metrics['packets_sent']) / interval,
            'packets_recv_per_sec': (new_metrics['packets_recv'] - old_metrics['packets_recv']) / interval,
            'errors_in_delta': new_metrics['errin'] - old_metrics['errin'],
            'errors_out_delta': new_metrics['errout'] - old_metrics['errout'],
            'drops_in_delta': new_metrics['dropin'] - old_metrics['dropin'],
            'drops_out_delta': new_metrics['dropout'] - old_metrics['dropout']
        }
        
    def check_thresholds(self, rates, metrics):
        """Check for threshold violations"""
        if not rates or not metrics:
            return
            
        # Check interface speed (if available)
        if metrics['speed'] and metrics['speed'] > 0:
            max_bytes_per_sec = metrics['speed'] * 1000000 / 8  # Convert Mbps to bytes/sec
            
            # Check utilization
            for direction in ['sent', 'recv']:
                utilization = rates[f'bytes_{direction}_per_sec'] / max_bytes_per_sec * 100
                if utilization > self.threshold:
                    self.alert_queue.put({
                        'type': 'high_utilization',
                        'direction': direction,
                        'value': utilization,
                        'threshold': self.threshold,
                        'timestamp': datetime.now().isoformat()
                    })
                    
        # Check error rates
        for error_type in ['errors_in_delta', 'errors_out_delta', 'drops_in_delta', 'drops_out_delta']:
            if rates[error_type] > 0:
                self.alert_queue.put({
                    'type': 'error_detected',
                    'error_type': error_type,
                    'value': rates[error_type],
                    'timestamp': datetime.now().isoformat()
                })
                
    def alert_handler(self):
        """Handle and log alerts"""
        while not self.stop_event.is_set():
            try:
                alert = self.alert_queue.get(timeout=1)
                if alert['type'] == 'high_utilization':
                    logging.warning(
                        f"High network utilization detected: "
                        f"{alert['direction']} = {alert['value']:.2f}% "
                        f"(threshold: {alert['threshold']}%)"
                    )
                elif alert['type'] == 'error_detected':
                    logging.warning(
                        f"Network error detected: {alert['error_type']} = {alert['value']}"
                    )
            except queue.Empty:
                continue
                
    def monitor(self):
        """Start network monitoring"""
        logging.info(f"Starting network monitoring on interface {self.interface_name}")
        logging.info(f"IP Address: {self.interface_ip}")
        logging.info(f"Monitoring interval: {self.interval} seconds")
        logging.info(f"Utilization threshold: {self.threshold}%")
        
        # Start alert handler thread
        alert_thread = threading.Thread(target=self.alert_handler)
        alert_thread.start()
        
        last_metrics = None
        start_time = time.time()
        
        try:
            while not self.stop_event.is_set():
                current_metrics = self.collect_interface_metrics()
                
                if last_metrics:
                    rates = self.calculate_rates(last_metrics, current_metrics, self.interval)
                    self.check_thresholds(rates, current_metrics)
                    
                    # Log current status
                    if rates:
                        logging.info(
                            f"Network Status - Upload: {rates['bytes_sent_per_sec']/1024:.2f} KB/s, "
                            f"Download: {rates['bytes_recv_per_sec']/1024:.2f} KB/s"
                        )
                        
                last_metrics = current_metrics
                time.sleep(self.interval)
                
        except KeyboardInterrupt:
            logging.info("Monitoring stopped by user")
        finally:
            self.stop_event.set()
            alert_thread.join()
            
            # Save final statistics
            self.save_statistics(start_time)
            
    def save_statistics(self, start_time):
        """Save monitoring statistics to file"""
        stats = {
            'start_time': datetime.fromtimestamp(start_time).isoformat(),
            'end_time': datetime.now().isoformat(),
            'duration': time.time() - start_time,
            'interface': self.interface_name,
            'ip_address': self.interface_ip,
            'final_metrics': self.collect_interface_metrics()
        }
        
        stats_file = Path("network_logs") / f"stats_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json"
        with open(stats_file, 'w') as f:
            json.dump(stats, f, indent=2)
            
        logging.info(f"Statistics saved to {stats_file}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Advanced Network Monitor")
    parser.add_argument("ip", help="Interface IP to monitor")
    parser.add_argument("--interval", type=float, default=1,
                      help="Monitoring interval in seconds")
    parser.add_argument("--threshold", type=float, default=80,
                      help="Utilization threshold percentage")
    args = parser.parse_args()
    
    monitor = NetworkMonitor(args.ip, args.interval, args.threshold)
    monitor.monitor()
